/***************************************************************************\
  "scope.h" - Scope Stack Module (Phase C)
  Revision 3; 10 Nov 2002 - 14 Nov 2002
  Copyright (C) 2002  Kevin P. Albrecht
\***************************************************************************/

#ifndef SCOPESTACK_H
#define SCOPESTACK_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "table.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

// External //
void scope_init ();
void scope_deinit ();
void scope_begin ();
void scope_end ();
Symbol *scope_local_enter (char *aKey);
Symbol *scope_local_search (char *aKey);
Symbol *scope_global_enter (char *aKey);
Symbol *scope_global_search (char *aKey);

// Internal //
Table **mScopeStack;
int mScopeCurr;

Symbol **mSymbolList;
int mSymbolListCurr;

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

void scope_init ()
{
  int i;

  mScopeCurr = 0;
  mSymbolListCurr = 0;

  // Allocate space for the scope stack //
  mScopeStack = (Table **) malloc (sizeof (Table *) * GLOBAL_MAX_SCOPES);
  for (i = 0; i < GLOBAL_MAX_SCOPES; i++)
  {
    mScopeStack[i] = NULL;
  }

  mScopeStack[mScopeCurr] = tab_new ();

  // Allocate space for the symbol list, used to free memory at end of program //
  mSymbolList = (Symbol **) malloc (sizeof (Symbol *) * GLOBAL_MAX_SYMBOLS);
  for (i = 0; i < GLOBAL_MAX_SYMBOLS; i++)
  {
    mSymbolList[i] = NULL;
  }  
}

void scope_deinit ()
{
  int i;

  free (mScopeStack);

  for (i = 0; mSymbolList[i] != NULL; i++)
  {
    sym_del (mSymbolList[i]);
  }
}

void scope_begin ()
{
  mScopeCurr++;

  // Check to see if maximum scopes reached //
  if (mScopeCurr >= GLOBAL_MAX_SCOPES)
  {
    err (true, EF_MAX_SCOPES_REACHED, conv_is (GLOBAL_MAX_SCOPES));
    erre (EFF_TERMINATE);
  }

  mScopeStack[mScopeCurr] = tab_new ();
}

void scope_end ()
{
  tab_del (mScopeStack[mScopeCurr]);

  mScopeCurr--;

  // Check to see if minimum scopes reached //
  if (mScopeCurr < 0)
  {
    err (true, EF_MIN_SCOPES_REACHED, NULL);
    erre (EFF_TERMINATE);
  }
}

Symbol *scope_local_enter (char *aKey)
{
  Symbol *temp;

  temp = tab_enter (mScopeStack[mScopeCurr], aKey);
  mSymbolList[mSymbolListCurr] = temp;
  mSymbolListCurr++;

  return temp;
}

Symbol *scope_local_search (char *aKey)
{
  Symbol *temp;
  int i;

  temp = tab_search (mScopeStack[mScopeCurr], aKey);
  i = mScopeCurr - 1;

  while ((!temp) && (i >= 0))
  {
    temp = tab_search (mScopeStack[i], aKey);
    i--;
  }

  return temp;
}

Symbol *scope_global_enter (char *aKey)
{
  Symbol *temp;

  temp = tab_enter (mScopeStack[0], aKey);
  mSymbolList[mSymbolListCurr] = temp;
  mSymbolListCurr++;

  return temp;
}

Symbol *scope_global_search (char *aKey)
{
  return tab_search (mScopeStack[0], aKey);
}

#endif
